# coding: utf-8
"""

    ファイル名：   BasicUtils.py

"""

from __future__ import division, print_function, unicode_literals

import sys
import os
import getpass
import time
import shutil

def mkdirs_with_retry( path, retry=3 ):
    retry_ = 0
    while( not os.path.exists( path ) ):
        try:
            os.makedirs( path )
        except:
            if retry_ < retry:
                retry_ += 1
                time.sleep( 1 )
                print( 'retry os.makedirs( %s )' % ( path ) )
            else:
                assert( False )

def clear_dirs_with_retry( dirs, retry=3  ):
    for dir_ in dirs:
        if os.path.exists( dir_ ):
            shutil.rmtree( dir_ )
        mkdirs_with_retry( dir_, retry=retry )

def rename_with_retry( old_path, new_path, retry=3 ):
    retry_ = 0
    done_ = False
    while( not done_ ):
        try:
            os.rename( old_path, new_path )
            done_ = True
        except:
            if retry_ < retry:
                retry_ += 1
                time.sleep( 1 )
                print( 'retry os.rename( %s, %s )' % ( old_path, new_path ) )
            else:
                assert( False )

def is_empty_dir( path ):
    if not os.path.exists( path ):
        return False

    for dirpath, dirnames, files in os.walk( path ):
        # print( 'files=', files )
        if files:
            return False
        else:
            return True

def exe_name():
    name_ =  os.path.split( sys.argv[0] )[-1]
    for ext in ( '.exe', '.py' ):
        name_ = name_.replace( ext, '' )
    return name_

def home_dir():
    username = getpass.getuser()
    return os.path.expanduser( '~' + username )

"""
    URL: http://stackoverflow.com/questions/1305532/convert-python-dict-to-object

    namedtuple の場合、メンバーの更新は can't set attritube により不可のようなので
    この Struct の方が柔軟。
"""
class Struct:
    def __init__(self, **entries): 
        self.__dict__.update(entries)

def make_indecies_text( indecies, offset=0 ):
    assert( len( indecies ) > 0 )
    previous = None
    is_continuous = True
    for i in indecies:
        if previous is not None:
            if i != previous + 1:
                is_continuous = False
        previous = i

    if is_continuous:
        if len( indecies ) == 1:
            text = str( offset+indecies[0] )
        else:
            text = '%d-%d' % ( offset+indecies[0], offset+indecies[-1] )
    else:
        text = ','.join( [ str(offset+i) for i in indecies ] )

    return text

"""
thanks to: http://stackoverflow.com/questions/9647202/ordinal-numbers-replacement
"""
def ordinal_str(numb):
    if numb < 20: #determining suffix for < 20
        if numb == 1: 
            suffix = 'st'
        elif numb == 2:
            suffix = 'nd'
        elif numb == 3:
            suffix = 'rd'
        else:
            suffix = 'th'  
    else:   #determining suffix for > 20
        tens = str(numb)
        tens = tens[-2]
        unit = str(numb)
        unit = unit[-1]
        if tens == "1":
           suffix = "th"
        else:
            if unit == "1": 
                suffix = 'st'
            elif unit == "2":
                suffix = 'nd'
            elif unit == "3":
                suffix = 'rd'
            else:
                suffix = 'th'
    return str(numb)+ suffix

"""
    from http://stackoverflow.com/questions/651794/whats-the-best-way-to-initialize-a-dict-of-dicts-in-python
"""

class AutoVivifiedDict(dict):
    """Implementation of perl's autovivification feature."""
    def __getitem__(self, item):
        try:
            return dict.__getitem__(self, item)
        except KeyError:
            value = self[item] = type(self)()
            return value
