# coding: utf-8
"""

    ファイル名：   DebugCanvas.py

    処理内容：

        デバッグ用の plot 

    Copyright (c) 2017, Masatsuyo Takahashi, KEK-PF

"""
import numpy                            as np
import matplotlib.pyplot                as plt
from matplotlib.backends.backend_tkagg  import FigureCanvasTkAgg
from OurTkinter                         import Tk, Dialog

class DebugCanvas( Dialog ):
    def __init__( self, title, draw_func, parent=None, figsize=None ):

        if parent is None:
            parent = Tk.Toplevel()
            parent.withdraw()
            self.created_parent = True
        else:
            self.created_parent = False

        self.grab = 'local'     # used in grab_set
        self.parent     = parent
        self.figsize    = figsize
        self.draw_func  = draw_func
        self.continue_  = None

        Dialog.__init__( self, parent, title ) # this calls body
        # TODO: self.resizable(width=False, height=False)

    def destroy_parent( self ):
        if self.created_parent:
            self.parent.destroy()

    def body( self, body_frame ):   # overrides parent class method

        cframe = Tk.Frame( body_frame )
        cframe.pack()

        figsize_ = ( 18, 9 ) if self.figsize is None else self.figsize

        fig = plt.figure( figsize=figsize_ )

        self.mpl_canvas = FigureCanvasTkAgg( fig, cframe )
        self.mpl_canvas.show()
        self.mpl_canvas_widget = self.mpl_canvas.get_tk_widget()
        self.mpl_canvas_widget.pack( fill=Tk.BOTH, expand=1 )

        # it seems that draw_func should be called after the creation of mpl_canvas
        # in order to enable 3d-rotation of the figure
        self.draw_func( fig )
        self.parent.config( cursor='' )

        self.protocol( "WM_DELETE_WINDOW", self.ok )

    def ok( self ):
        self.destroy_parent()
        self.continue_  = True
        self.destroy()

    def cancel( self ):
        self.destroy_parent()
        self.continue_  = False
        self.destroy()

